@@ -1,38 +1,41 @@
-from __future__ import unicode_literals
 from __future__ import absolute_import
+from __future__ import unicode_literals
 
-import logging
-import collections
+
+# load python 3, fallback to python 2 if it fails
 try:
-    from urllib import unquote, unquote_plus, quote_plus
-except ImportError:
     from urllib.parse import unquote, unquote_plus, quote_plus
+except ImportError:
+    from urllib import unquote, unquote_plus, quote_plus  # type: ignore
 from datetime import datetime, timedelta
 from itertools import tee
-
+import sys
 from flask import (
-    Flask, render_template, abort, url_for,
-    Response, stream_with_context, redirect,
-    request, session, jsonify
+    render_template, abort, url_for,
+    Response, stream_with_context, request, session, jsonify
 )
-from jinja2.utils import contextfunction
 
-from pypuppetdb.QueryBuilder import *
+import logging
+
+from pypuppetdb.QueryBuilder import (ExtractOperator, AndOperator,
+                                     EqualsOperator, FunctionOperator,
+                                     NullOperator, OrOperator,
+                                     LessEqualOperator, RegexOperator)
 
 from puppetboard.forms import ENABLED_QUERY_ENDPOINTS, QueryForm
 from puppetboard.utils import (get_or_abort, yield_or_stop,
                                get_db_version)
 from puppetboard.dailychart import get_daily_reports_chart
 
-import werkzeug.exceptions as ex
-import CommonMark
+try:
+    import CommonMark as commonmark
+except ImportError:
+    import commonmark
 
 from puppetboard.core import get_app, get_puppetdb, environments
-import puppetboard.errors
 
 from . import __version__
 
-
 REPORTS_COLUMNS = [
     {'attr': 'end', 'filter': 'end_time',
      'name': 'End time', 'type': 'datetime'},
@@ -50,7 +53,6 @@ CATALOGS_COLUMNS = [
     {'attr': 'form', 'name': 'Compare'},
 ]
 
-
 app = get_app()
 graph_facts = app.config['GRAPH_FACTS']
 numeric_level = getattr(logging, app.config['LOGLEVEL'].upper(), None)
@@ -79,11 +81,32 @@ def check_env(env, envs):
         abort(404)
 
 
+def metric_params(db_version):
+    query_type = ''
+
+    # Puppet Server is enforcing new metrics API (v2)
+    # starting with versions 6.9.1, 5.3.12, and 5.2.13
+    if (db_version > (6, 9, 0) or
+            (db_version > (5, 3, 11) and db_version < (6, 0, 0)) or
+            (db_version > (5, 2, 12) and db_version < (5, 3, 10))):
+        metric_version = 'v2'
+    else:
+        metric_version = 'v1'
+
+    # Puppet DB version changed the query format from 3.2.0
+    # to 4.0 when querying mbeans
+    if db_version < (4, 0, 0):
+        query_type = 'type=default,'
+
+    return query_type, metric_version
+
+
 @app.context_processor
 def utility_processor():
     def now(format='%m/%d/%Y %H:%M:%S'):
         """returns the formated datetime"""
-        return datetime.datetime.now().strftime(format)
+        return datetime.now().strftime(format)
+
     return dict(now=now)
 
 
@@ -107,31 +130,35 @@ def index(env):
         query = app.config['OVERVIEW_FILTER']
 
         prefix = 'puppetlabs.puppetdb.population'
-        query_type = ''
-
-        # Puppet DB version changed the query format from 3.2.0
-        # to 4.0 when querying mbeans
-        if get_db_version(puppetdb) < (4, 0, 0):
-            query_type = 'type=default,'
+        db_version = get_db_version(puppetdb)
+        query_type, metric_version = metric_params(db_version)
 
         num_nodes = get_or_abort(
             puppetdb.metric,
-            "{0}{1}".format(prefix, ':%sname=num-nodes' % query_type))
+            "{0}{1}".format(prefix, ':%sname=num-nodes' % query_type),
+            version=metric_version)
         num_resources = get_or_abort(
             puppetdb.metric,
-            "{0}{1}".format(prefix, ':%sname=num-resources' % query_type))
+            "{0}{1}".format(prefix, ':%sname=num-resources' % query_type),
+            version=metric_version)
         avg_resources_node = get_or_abort(
             puppetdb.metric,
             "{0}{1}".format(prefix,
-                            ':%sname=avg-resources-per-node' % query_type))
+                            ':%sname=avg-resources-per-node' % query_type),
+            version=metric_version)
         metrics['num_nodes'] = num_nodes['Value']
         metrics['num_resources'] = num_resources['Value']
-        metrics['avg_resources_node'] = "{0:10.0f}".format(
-            avg_resources_node['Value'])
+        try:
+            # Compute our own average because avg_resources_node['Value']
+            # returns a string of the format "num_resources/num_nodes"
+            # example: "1234/9" instead of doing the division itself.
+            metrics['avg_resources_node'] = "{0:10.0f}".format(
+                (num_resources['Value'] / num_nodes['Value']))
+        except ZeroDivisionError:
+            metrics['avg_resources_node'] = 0
     else:
         query = AndOperator()
         query.add(EqualsOperator('catalog_environment', env))
-        query.add(EqualsOperator('facts_environment', env))
 
         num_nodes_query = ExtractOperator()
         num_nodes_query.add_field(FunctionOperator('count'))
@@ -223,12 +250,11 @@ def nodes(env):
 
     if env != '*':
         query.add(EqualsOperator("catalog_environment", env))
-        query.add(EqualsOperator("facts_environment", env))
 
     if status_arg in ['failed', 'changed', 'unchanged']:
         query.add(EqualsOperator('latest_report_status', status_arg))
     elif status_arg == 'unreported':
-        unreported = datetime.datetime.utcnow()
+        unreported = datetime.utcnow()
         unreported = (unreported -
                       timedelta(hours=app.config['UNRESPONSIVE_HOURS']))
         unreported = unreported.replace(microsecond=0).isoformat()
@@ -549,7 +575,7 @@ def report(env, node_name, report_id):
     except StopIteration:
         abort(404)
 
-    report.version = CommonMark.commonmark(report.version)
+    report.version = commonmark.commonmark(report.version)
 
     return render_template(
         'report.html',
@@ -723,7 +749,7 @@ def fact_ajax(env, node, fact, value):
                 fact_h.node))
         if not value:
             fact_value = fact_h.value
-            if isinstance(fact_value, unicode) or isinstance(fact_value, str):
+            if isinstance(fact_value, str):
                 fact_value = quote_plus(fact_h.value)
 
             line.append('<a href="{0}">{1}</a>'.format(
@@ -811,9 +837,46 @@ def metrics(env):
     envs = environments()
     check_env(env, envs)
 
-    metrics = get_or_abort(puppetdb._query, 'mbean')
+    db_version = get_db_version(puppetdb)
+    query_type, metric_version = metric_params(db_version)
+    if metric_version == 'v1':
+        mbeans = get_or_abort(puppetdb._query, 'mbean')
+        metrics = list(mbeans.keys())
+    elif metric_version == 'v2':
+        # the list response is a dict in the format:
+        # {
+        #   "domain1": {
+        #     "property1": {
+        #      ...
+        #     }
+        #   },
+        #   "domain2": {
+        #     "property2": {
+        #      ...
+        #     }
+        #   }
+        # }
+        # The MBean names are the combination of the domain and the properties
+        # with a ":" in between, example:
+        #   domain1:property1
+        #   domain2:property2
+        # reference: https://jolokia.org/reference/html/protocol.html#list
+        metrics_domains = get_or_abort(puppetdb.metric)
+        metrics = []
+        # get all of the domains
+        for domain in list(metrics_domains.keys()):
+            # iterate over all of the properties in this domain
+            properties = list(metrics_domains[domain].keys())
+            for prop in properties:
+                # combine the current domain and each property with
+                # a ":" in between
+                metrics.append(domain + ':' + prop)
+    else:
+        raise ValueError("Unknown metric version {} for database version {}"
+                         .format(metric_version, database_version))
+
     return render_template('metrics.html',
-                           metrics=sorted(metrics.keys()),
+                           metrics=sorted(metrics),
                            envs=envs,
                            current_env=env)
 
@@ -831,8 +894,11 @@ def metric(env, metric):
     envs = environments()
     check_env(env, envs)
 
+    db_version = get_db_version(puppetdb)
+    query_type, metric_version = metric_params(db_version)
+
     name = unquote(metric)
-    metric = get_or_abort(puppetdb.metric, metric)
+    metric = get_or_abort(puppetdb.metric, metric, version=metric_version)
     return render_template(
         'metric.html',
         name=name,
@@ -858,7 +924,7 @@ def catalogs(env, compare):
     check_env(env, envs)
 
     if not app.config['ENABLE_CATALOG']:
-        log.warn('Access to catalog interface disabled by administrator')
+        log.warning('Access to catalog interface disabled by administrator')
         abort(403)
 
     return render_template(
@@ -995,20 +1061,20 @@ def radiator(env):
     check_env(env, envs)
 
     if env == '*':
-        query_type = ''
-        if get_db_version(puppetdb) < (4, 0, 0):
-            query_type = 'type=default,'
+        db_version = get_db_version(puppetdb)
+        query_type, metric_version = metric_params(db_version)
+
         query = None
         metrics = get_or_abort(
             puppetdb.metric,
-            'puppetlabs.puppetdb.population:%sname=num-nodes' % query_type)
+            'puppetlabs.puppetdb.population:%sname=num-nodes' % query_type,
+            version=metric_version)
         num_nodes = metrics['Value']
     else:
         query = AndOperator()
         metric_query = ExtractOperator()
 
         query.add(EqualsOperator("catalog_environment", env))
-        query.add(EqualsOperator("facts_environment", env))
         metric_query.add_field(FunctionOperator('count'))
         metric_query.add_query(query)
 
