@@ -1,17 +1,13 @@
-from __future__ import unicode_literals
 from __future__ import absolute_import
+from __future__ import unicode_literals
 
 from collections import OrderedDict
 
 from flask_wtf import FlaskForm
-from wtforms import (
-    HiddenField, RadioField, SelectField,
-    TextAreaField, BooleanField, validators
-)
+from wtforms import (BooleanField, RadioField, TextAreaField, validators)
 
 from puppetboard.core import get_app
 
-
 app = get_app()
 QUERY_ENDPOINTS = OrderedDict([
     # PuppetDB API endpoint, Form name
@@ -35,7 +31,7 @@ ENABLED_QUERY_ENDPOINTS = app.config.get(
 class QueryForm(FlaskForm):
     """The form used to allow freeform queries to be executed against
     PuppetDB."""
-    query = TextAreaField('Query', [validators.Required(
+    query = TextAreaField('Query', [validators.DataRequired(
         message='A query is required.')])
     endpoints = RadioField('API endpoint', choices=[
         (key, value) for key, value in QUERY_ENDPOINTS.items()
